@@ -6,12 +6,14 @@ from maintain_PlatoUtils.maintain_PlatoUtils import wrapNebula2Df
 import tqdm
 from typing import Callable, Any, Iterable
 import traceback
+from elasticsearch import Elasticsearch
 
 class NodeInfo:
-    def __init__(self,nodeType,nodeIdAttr,nodeIdVal=None):
+    def __init__(self,nodeType,nodeIdAttr,nodeIdVal="",nodeIdVal_fuzzy=""):
         self.nodeType=nodeType
         self.nodeIdAttr=nodeIdAttr
         self.nodeIdVal=nodeIdVal
+        self.nodeIdVal_fuzzy=nodeIdVal_fuzzy
         self.nodeInfo=self.__dict__
         
         
@@ -33,11 +35,9 @@ class RDFInfo:
 
 class GraphWrapClient:
     
-    def __init__(self,gHost,gPort,gUser,gPassword,gDbName=""):
+    def __init__(self,gClient,gDbName=""):
         '''初始化gClient'''
-        Connection_pool = ConnectionPool(gHost, gPort,network_timeout=300000)
-        self.gClient = GraphClient(Connection_pool)
-        self.gClient.authenticate(gUser, gPassword)
+        self.gClient=gClient
         
         if len(gDbName)>0:
             self.gClient.execute_query("USE {}".format(gDbName))
@@ -90,9 +90,11 @@ class GraphWrapClient:
         
 class GraphWrapQuery:
     
-    def __init__(self):
+    def __init__(self,esClient=None,esIndex=""):
         self.yieldAttrList=[]
         self.yieldSysIdList=[]
+        self.esClient=esClient
+        self.esIndex=esIndex
     
     def singleSearchFunc_wait(self,hetDict,head=True,singleYieldList={}):
         args=[hetDict]
@@ -131,13 +133,77 @@ class GraphWrapQuery:
                 singleYieldDict[nodeTypeAttrItem["nodeType"]]=singleYieldDict.get(nodeTypeAttrItem["nodeType"],[])+[nodeTypeAttrItem["nodeAttr"]]
             
         # LOOKUP语句构建
-        if head==True:  
-            singleQueryStrList=["LOOKUP ON {nodeType} WHERE {nodeType}.{nodeIdAttr}=='{nodeIdVal}'|\
-                                        YIELD $-.VertexID AS src{nodeType}SysId".format(
-                                                                                            nodeType=headInfo["nodeType"],
-                                                                                            nodeIdAttr=headInfo["nodeIdAttr"],
-                                                                                            nodeIdVal=headInfo["nodeIdVal"]
-                                                                                        )]
+        if head==True:
+            
+            if len(headInfo.get("nodeIdVal_fuzzy",""))>0:
+                if self.esClient!=None:
+                    qbody={
+                            "query":{
+                                "bool":{
+                                    "should":[
+                                        {"match":{
+                                            "Name.keyword": {
+                                                "query": headInfo["nodeIdVal_fuzzy"],  
+                                                "fuzziness": "AUTO",
+                                                "prefix_length": 0,
+                                                "max_expansions":1,
+                                                "boost":10
+                                            }
+                                        }},
+                                        {"match": {
+                                            "AliasNames.keyword": {
+                                                "query": headInfo["nodeIdVal_fuzzy"],
+                                                "fuzziness": "AUTO",
+                                                "prefix_length": 0,
+                                                "max_expansions":1,
+                                                "boost":1
+                                            }
+                                        }}
+                                    ]
+                                }
+                            }
+                        }
+                    if len(headInfo.get("nodeType",""))>0:
+                        nodeTypeESJudgeJson={
+                            "term":{
+                                "gTagName":headInfo["nodeType"]
+                            }
+                        }
+                        qbody["query"]["bool"]["must"]=qbody["query"]["bool"].get("must",[])+[nodeTypeESJudgeJson]
+                    
+                        
+                    esResult=self.esClient.search(index=self.esIndex,body=qbody,timeout='60s')["hits"]["hits"]
+                    if len(esResult)>0:
+                        if len(esResult)>0:
+                            rebuiltName=esResult[0]["_source"]["Name"]
+                        elif len(headInfo.get("nodeIdVal",""))>0:
+                            rebuiltName=headInfo["nodeIdVal"]
+                        else:
+                            raise Exception("无法找到符合名称'{}'的'{}'类型节点".format(headInfo["nodeIdVal_fuzzy"],headInfo["nodeType"]))
+                    else:
+                        raise Exception("无法找到'{}'相关信息".format(headInfo["nodeIdVal_fuzzy"]))
+                    headInfo["nodeIdVal"]=rebuiltName
+                     
+                    for keyItem in esResult[0]["_source"]:
+                        if esResult[0]["_source"][keyItem]==esResult[0]["_source"]["Name"] and keyItem!="Name":
+                            headInfo["nodeType"]=esResult[0]["_source"]["gTagName"]
+                            headInfo["nodeIdAttr"]=keyItem
+            
+            if headInfo["nodeIdVal"]!="*":
+                singleQueryStrList=["LOOKUP ON {nodeType} WHERE {nodeType}.{nodeIdAttr}=='{nodeIdVal}'|\
+                                            YIELD $-.VertexID AS src{nodeType}SysId".format(
+                                                                                                nodeType=headInfo["nodeType"],
+                                                                                                nodeIdAttr=headInfo["nodeIdAttr"],
+                                                                                                nodeIdVal=headInfo["nodeIdVal"]
+                                                                                            )]
+                
+            else:
+                singleQueryStrList=["LOOKUP ON {nodeType} WHERE {nodeType}.{nodeIdAttr}!='不可能的名字'|\
+                                            YIELD $-.VertexID AS src{nodeType}SysId".format(
+                                                                                                nodeType=headInfo["nodeType"],
+                                                                                                nodeIdAttr=headInfo["nodeIdAttr"]
+                                                                                            )]
+                
             srcSysIdName="src{}SysId".format(headInfo["nodeType"])
             self.yieldSysIdList.append(srcSysIdName)
         
@@ -146,28 +212,52 @@ class GraphWrapQuery:
         pureAttrYieldList=[]
         for singleYieldTypeKey in singleYieldDict:
             for singleYieldAttrVal in singleYieldDict[singleYieldTypeKey]:
-                nodeType=singleYieldTypeKey
-                nodeAttr=singleYieldAttrVal
-                nodeTypeAttrName="{nodeType}{nodeAttr}".format(nodeType=nodeType,
-                                                            nodeAttr=nodeAttr)
-                if nodeTypeAttrName not in self.yieldAttrList:
-                    if len(headInfo)>0 and headInfo["nodeType"]==singleYieldTypeKey:
-                        yieldAttrList.append("$^.{nodeType}.{nodeAttr} AS {nodeType}{nodeAttr}".format(nodeType=nodeType,
-                                                                                                    nodeAttr=nodeAttr))
-                        if nodeTypeAttrItem not in self.yieldAttrList:
-                            pureAttrYieldList.append(nodeTypeAttrName)
-                    if len(edgeInfo)>0 and tailInfo["nodeType"]==singleYieldTypeKey:
-                        yieldAttrList.append("$$.{nodeType}.{nodeAttr} AS {nodeType}{nodeAttr}".format(nodeType=nodeType,
-                                                                                                    nodeAttr=nodeAttr))
-                        if nodeTypeAttrItem not in self.yieldAttrList:
+                
+                if len(headInfo)>0 and headInfo["nodeType"]==singleYieldTypeKey:
+                    nodeType=singleYieldTypeKey
+                    nodeAttr=singleYieldAttrVal
+                    nodeTypeAttrName="{nodeType}{nodeAttr}".format(nodeType=nodeType,
+                                                                nodeAttr=nodeAttr)
+                    
+                    oriNodeTypeAttrName=nodeTypeAttrName
+                    tmpNodeTypeAttrName=oriNodeTypeAttrName+"_0"
+                    tailI=1
+                    while tmpNodeTypeAttrName in self.yieldAttrList+pureAttrYieldList:
+                        tmpNodeTypeAttrName=oriNodeTypeAttrName+"_"+str(tailI)
+                        tailI+=1
+                    nodeTypeAttrName=tmpNodeTypeAttrName
+                    if head==True:
+                        yieldAttrList.append("$^.{nodeType}.{nodeAttr} AS {nodeTypeAttrName}".format(nodeType=nodeType,
+                                                                                                        nodeAttr=nodeAttr,
+                                                                                                    nodeTypeAttrName=nodeTypeAttrName))
+                        if nodeTypeAttrItem not in self.yieldAttrList+pureAttrYieldList:
                             pureAttrYieldList.append(nodeTypeAttrName)
+                if len(edgeInfo)>0 and tailInfo["nodeType"]==singleYieldTypeKey:
+                    nodeType=singleYieldTypeKey
+                    nodeAttr=singleYieldAttrVal
+                    nodeTypeAttrName="{nodeType}{nodeAttr}".format(nodeType=nodeType,
+                                                                nodeAttr=nodeAttr)
+                    
+                    oriNodeTypeAttrName=nodeTypeAttrName
+                    tmpNodeTypeAttrName=oriNodeTypeAttrName+"_0"
+                    tailI=1
+                    while tmpNodeTypeAttrName in self.yieldAttrList+pureAttrYieldList:
+                        tmpNodeTypeAttrName=oriNodeTypeAttrName+"_"+str(tailI)
+                        tailI+=1
+                    nodeTypeAttrName=tmpNodeTypeAttrName
+                    
+                    yieldAttrList.append("$$.{nodeType}.{nodeAttr} AS {nodeTypeAttrName}".format(nodeType=nodeType,
+                                                                                                    nodeAttr=nodeAttr,
+                                                                                                nodeTypeAttrName=nodeTypeAttrName))
+                    if nodeTypeAttrItem not in self.yieldAttrList+pureAttrYieldList:
+                        pureAttrYieldList.append(nodeTypeAttrName)
                         
         oldYieldAttrList=[]
         for yieldAttrItem in self.yieldAttrList:
             oldYieldAttrList.append("$-.{oldYieldAttr} AS {oldYieldAttr}".format(oldYieldAttr=yieldAttrItem))
         yieldAttrListStr=""
         if len(yieldAttrList)>0:
-            yieldAttrListStr=","+",".join(yieldAttrList+oldYieldAttrList)
+            yieldAttrListStr=","+",".join(oldYieldAttrList+yieldAttrList)
 
         yieldSysIdList=["$-.{yieldSysId} AS {yieldSysId}".format(yieldSysId=yieldSysIdItem) for yieldSysIdItem in self.yieldSysIdList]
         yieldSysIdListStr=""
@@ -178,29 +268,39 @@ class GraphWrapQuery:
         
         
         # GO语句构建-完整化
-        if len(edgeInfo)>0:
-            # RDF搜索
-            if len(tailInfo)>0:
+        # RDF搜索
+        if len(tailInfo)>0:
+            tgtSysIdName="tgt{}SysId".format(tailInfo["nodeType"])
+            oriTgtSysIdName=tgtSysIdName
+            tmpTgtSysIdName=oriTgtSysIdName+"_0"
+            tailI=1
+            while tmpTgtSysIdName in self.yieldSysIdList:
+                tmpTgtSysIdName=oriTgtSysIdName+"_"+str(tailI)
+                tailI+=1
+            tgtSysIdName=tmpTgtSysIdName
+            
+            if len(edgeInfo)>0:
                 startSysId=self.yieldSysIdList[-1]
-                singleQueryStrList.append("GO FROM $-.{startSysId} OVER {edgeType} {direct} YIELD {edgeType}._dst AS tgt{nodeType}SysId {yieldList}{yieldAttrList}".format(
+                singleQueryStrList.append("GO FROM $-.{startSysId} OVER {edgeType} {direct} YIELD {edgeType}._dst AS {tgtSysIdName} {yieldList}{yieldAttrList}".format(
                                                         startSysId=startSysId,
                                                         edgeType=edgeInfo["edgeType"],
                                                         direct=edgeInfo["direct"],
+                                                        tgtSysIdName=tgtSysIdName,
                                                         nodeType=tailInfo["nodeType"],
                                                         yieldList=yieldSysIdListStr,
                                                         yieldAttrList=yieldAttrListStr
                                                     ))
             else:
-                singleQueryStrList.append("GO FROM $-.VertexID OVER {edgeType} {direct} YIELD {edgeType}._dst AS tgt{nodeType}SysId {yieldList}{yieldAttrList}".format(
+                singleQueryStrList.append("GO FROM $-.VertexID OVER {edgeType} {direct} YIELD {edgeType}._dst AS {tgtSysIdName} {yieldList}{yieldAttrList}".format(
                                                         edgeType=edgeInfo["edgeType"],
                                                         direct=edgeInfo["direct"],
+                                                        tgtSysIdName=tgtSysIdName,
                                                         nodeType=tailInfo["nodeType"],
                                                         yieldList=yieldSysIdListStr,
                                                         yieldAttrList=yieldAttrListStr
                                                     ))
         
             # 整理新的yield列表
-            tgtSysIdName="tgt{}SysId".format(tailInfo["nodeType"])
             self.yieldSysIdList.append(tgtSysIdName)
             self.yieldAttrList+=pureAttrYieldList
             # self.yieldAttrList=list(set(self.yieldAttrList))
@@ -362,8 +462,12 @@ if __name__=="__main__":
     gPassword="nebula"
     gDbName="company_product_field_musklin"
 
-    myGClient=GraphWrapClient(gHost,gPort,gUser,gPassword,gDbName=gDbName)
-    myGQuery=GraphWrapQuery()
+    Connection_pool = ConnectionPool(gHost, gPort,network_timeout=300000)
+    gClient = GraphClient(Connection_pool)
+    gClient.authenticate(gUser, gPassword)
+    myGClient=GraphWrapClient(gClient,gDbName=gDbName)
+    esClient= Elasticsearch(["9.134.92.196:9200"],http_auth=("elastic", "devcloud@123"))
+    myGQuery=GraphWrapQuery(esClient=esClient,esIndex="hr_integrate_kg")
     
     # # single vertex search with singleSearch
     # queryStr=myGQuery.singleSearchFunc(NodeInfo("Company","CompanyName",nodeIdVal="深圳市腾讯计算机系统有限公司"),
@@ -471,85 +575,168 @@ if __name__=="__main__":
     # )
     # myGQuery.renew()
     
-    # # wrapJson2Qeury-1
-    # queryJson={
-    #         "searchMethod":"seqSearch",
-    #         "searchFrame":[
-    #             {
-    #                 "searchMethod":"singleSearch",
-    #                 "searchFrame":{
-    #                     "head":{
-    #                         "nodeType":"Company",
-    #                         "nodeIdAttr":"CompanyName",
-    #                         "nodeIdVal":"深圳市腾讯计算机系统有限公司"
-    #                     },
-    #                     "edge":{
-    #                         "edgeType":"produce"
-    #                     },
-    #                     "tail":{
-    #                         "nodeType":"Product",
-    #                         "nodeIdAttr":"ProductName",
-    #                         "nodeIdVal":None
-    #                     },
-    #                     "singleYieldList":[
-    #                         {"nodeType":"Company","nodeAttr":"CompanyName"},
-    #                         {"nodeType":"Product","nodeAttr":"ProductName"}
-    #                     ]
-    #                 }
-    #             },{
-    #                 "searchMethod":"singleSearch",
-    #                 "searchFrame":{
-    #                     "head":{
-    #                         "nodeType":"Product",
-    #                         "nodeIdAttr":"ProductName",
-    #                         "nodeIdVal":None
-    #                     },
-    #                     "edge":{
-    #                         "edgeType":"belongTo"
-    #                     },
-    #                     "tail":{
-    #                         "nodeType":"Field",
-    #                         "nodeIdAttr":"FieldName",
-    #                         "nodeIdVal":None
-    #                     },
-    #                     "singleYieldList":[
-    #                         {"nodeType":"Company","nodeAttr":"CompanyName"},
-    #                         {"nodeType":"Product","nodeAttr":"ProductName"},
-    #                         {"nodeType":"Field","nodeAttr":"FieldName"}
-    #                     ]
-    #                 }
-    #             }
-    #         ]
-    #     }
-    # queryFunc=myGQuery.wrapJson2Query(queryJson)
-    # queryStr=myGQuery.runWaitFunc(queryFunc)
-    
-    # wrapJson2Qeury-2
+    # wrapJson2Qeury-1
     queryJson={
-        "searchMethod":"singleSearch",
-        "searchFrame":{
-            "head":{
-                "nodeType":"Company",
-                "nodeIdAttr":"CompanyName",
-                "nodeIdVal":"深圳市腾讯计算机系统有限公司"
-            },
-            "edge":{
-                "edgeType":"produce"
-            },
-            "tail":{
-                "nodeType":"Product",
-                "nodeIdAttr":"ProductName",
-                "nodeIdVal":None
-            },
-            "singleYieldList":[
-                {"nodeType":"Company","nodeAttr":"CompanyName"},
-                {"nodeType":"Product","nodeAttr":"ProductName"}
+            "searchMethod":"seqSearch",
+            "searchFrame":[
+                {
+                    "searchMethod":"singleSearch",
+                    "searchFrame":{
+                        "head":{
+                            "nodeType":"Company",
+                            "nodeIdAttr":"CompanyName",
+                            "nodeIdVal":"深圳市腾讯计算机系统有限公司"
+                        },
+                        "edge":{
+                            "edgeType":"produce"
+                        },
+                        "tail":{
+                            "nodeType":"Product",
+                            "nodeIdAttr":"ProductName",
+                            "nodeIdVal":None
+                        },
+                        "singleYieldList":[
+                            {"nodeType":"Company","nodeAttr":"CompanyName"},
+                            {"nodeType":"Product","nodeAttr":"ProductName"}
+                        ]
+                    }
+                },{
+                    "searchMethod":"singleSearch",
+                    "searchFrame":{
+                        "head":{
+                            "nodeType":"Product",
+                            "nodeIdAttr":"ProductName",
+                            "nodeIdVal":None
+                        },
+                        "edge":{
+                            "edgeType":"belongTo"
+                        },
+                        "tail":{
+                            "nodeType":"Field",
+                            "nodeIdAttr":"FieldName",
+                            "nodeIdVal":None
+                        },
+                        "singleYieldList":[
+                            {"nodeType":"Company","nodeAttr":"CompanyName"},
+                            {"nodeType":"Product","nodeAttr":"ProductName"},
+                            {"nodeType":"Field","nodeAttr":"FieldName"}
+                        ]
+                    }
+                }
             ]
         }
-    }
     queryFunc=myGQuery.wrapJson2Query(queryJson)
     queryStr=myGQuery.runWaitFunc(queryFunc)
     
+    # # wrapJson2Qeury-2
+    # queryJson={
+    #     "searchMethod":"singleSearch",
+    #     "searchFrame":{
+    #         "head":{
+    #             "nodeType":"Company",
+    #             "nodeIdAttr":"CompanyName",
+    #             "nodeIdVal":"深圳市腾讯计算机系统有限公司"
+    #         },
+    #         "edge":{
+    #             "edgeType":"produce"
+    #         },
+    #         "tail":{
+    #             "nodeType":"Product",
+    #             "nodeIdAttr":"ProductName",
+    #             "nodeIdVal":None
+    #         },
+    #         "singleYieldList":[
+    #             {"nodeType":"Company","nodeAttr":"CompanyName"},
+    #             {"nodeType":"Product","nodeAttr":"ProductName"}
+    #         ]
+    #     }
+    # }
+    # queryFunc=myGQuery.wrapJson2Query(queryJson)
+    # queryStr=myGQuery.runWaitFunc(queryFunc)
+    
+    # wrapJson2Query-seq
+    queryJson={
+        "searchMethod":"seqSearch",
+        "searchFrame":[
+            {
+                "searchMethod":"singleSearch",
+                "searchFrame":{
+                    "head":{
+                        "nodeType":"YunPost",
+                        "nodeIdAttr":"YunPostName",
+                        "nodeIdVal":"*"
+                    },
+                    "edge":{
+                        "edgeType":"subPost"
+                    },
+                    "tail":{
+                        "nodeType":"YunPost",
+                        "nodeIdAttr":"YunPostName",
+                        "nodeIdVal":None
+                    },
+                    "singleYieldList":[
+                        {"nodeType":"YunPost","nodeAttr":"YunPostName"}
+                    ]
+                }
+            },{
+                "searchMethod":"singleSearch",
+                "searchFrame":{
+                    "head":{
+                        "nodeType":"YunPost",
+                        "nodeIdAttr":"YunPostName",
+                        "nodeIdVal":None
+                    },
+                    "edge":{
+                        "edgeType":"subPost"
+                    },
+                    "tail":{
+                        "nodeType":"YunPost",
+                        "nodeIdAttr":"YunPostName",
+                        "nodeIdVal":None
+                    },
+                    "singleYieldList":[
+                        {"nodeType":"YunPost","nodeAttr":"YunPostName"}
+                    ]
+                }
+            }
+        ]
+    }
+    # queryFunc=myGQuery.wrapJson2Query(queryJson)
+    # queryStr=myGQuery.runWaitFunc(queryFunc)
+    
+    # # singlesearch
+    # queryJson={
+    #     "searchMethod":"singleSearch",
+    #     "searchFrame":{
+    #         "head":{
+    #             "nodeType":"Company",
+    #             "nodeIdAttr":"CompanyName",
+    #             "nodeIdVal":"阿里巴巴"
+    #         },
+    #         "singleYieldList":[
+    #             {"nodeType":"Company","nodeAttr":"CompanyName"},
+    #             {"nodeType":"Company","nodeAttr":"AliasNames"}
+    #         ]
+    #     }
+    # }
+    # queryFunc=myGQuery.wrapJson2Query(queryJson)
+    # queryStr=myGQuery.runWaitFunc(queryFunc)
+    
+    # # fuzzy search
+    # queryJson={
+    #     "searchMethod":"singleSearch",
+    #     "searchFrame":{
+    #         "head":{
+    #             "nodeIdVal_fuzzy":"深圳市腾讯计算机系统有限公司"
+    #         },
+    #         "singleYieldList":[
+    #             {"nodeType":"Company","nodeAttr":"CompanyName"},
+    #             {"nodeType":"Company","nodeAttr":"AliasNames"}
+    #         ]
+    #     }
+    # }
+    # queryFunc=myGQuery.wrapJson2Query(queryJson)
+    # queryStr=myGQuery.runWaitFunc(queryFunc)
     
     print(queryStr)
     print(123)
