@@ -9,8 +9,8 @@ from codecs import encode
 import numpy as np
 import argparse
 import joblib
-# from maintain_PlatoUtils.maintain_PlatoUtils import existTag,wrapNebula2Df,existEdge,delEdge,delVertex
-from maintain_PlatoUtils import existTag,wrapNebula2Df,existEdge,delEdge,delVertex
+from maintain_PlatoUtils.maintain_PlatoUtils import existTag,wrapNebula2Df,existEdge,delEdge,delVertex
+# from maintain_PlatoUtils import existTag,wrapNebula2Df,existEdge,delEdge,delVertex
 import joblib
 import sys
 import os.path as osp
